@@ -2,12 +2,10 @@ package ByteLoader;
 
 use XSLoader ();
 
-$VERSION = 0.04;
+our $VERSION = '0.05';
 
 XSLoader::load 'ByteLoader', $VERSION;
 
-# Preloaded methods go here.
-
 1;
 __END__
 
@@ -20,18 +18,20 @@ ByteLoader - load byte compiled perl code
   use ByteLoader 0.04;
   <byte code>
 
-  use ByteLoader 0.04;
-  <byte code>
+  or just
+
+  perl -MByteLoader bytecode_file
 
 =head1 DESCRIPTION
 
-This module is used to load byte compiled perl code. It uses the source
-filter mechanism to read the byte code and insert it into the compiled
-code at the appropriate point.
+This module is used to load byte compiled perl code as produced by
+C<perl -MO=Bytecode=...>. It uses the source filter mechanism to read
+the byte code and insert it into the compiled code at the appropriate point.
 
 =head1 AUTHOR
 
 Tom Hughes <tom@compton.nu> based on the ideas of Tim Bunce and others.
+Many changes by Enache Adrian <enache@rdslink.ro> 2003 a.d.
 
 =head1 SEE ALSO
 
