@@ -51,7 +51,7 @@ these classes with methods that are specific to file handles.
 
 =item new ( FILENAME [,MODE [,PERMS]] )
 
-Creates a C<IO::File>.  If it receives any parameters, they are passed to
+Creates an C<IO::File>.  If it receives any parameters, they are passed to
 the method C<open>; if the open fails, the object is destroyed.  Otherwise,
 it is returned to the caller.
 
@@ -71,6 +71,8 @@ Otherwise, it is returned to the caller.
 
 =item open( FILENAME [,MODE [,PERMS]] )
 
+=item open( FILENAME, IOLAYERS )
+
 C<open> accepts one, two or three parameters.  With one parameter,
 it is just a front end for the built-in C<open> function.  With two or three
 parameters, the first parameter is a filename that may include
@@ -78,13 +80,16 @@ whitespace or other special characters, and the second parameter is
 the open mode, optionally followed by a file permission value.
 
 If C<IO::File::open> receives a Perl mode string ("E<gt>", "+E<lt>", etc.)
-or a ANSI C fopen() mode string ("w", "r+", etc.), it uses the basic
+or an ANSI C fopen() mode string ("w", "r+", etc.), it uses the basic
 Perl C<open> operator (but protects any special characters).
 
 If C<IO::File::open> is given a numeric mode, it passes that mode
 and the optional permissions value to the Perl C<sysopen> operator.
 The permissions default to 0666.
 
+If C<IO::File::open> is given a mode that includes the C<:> character,
+it passes all the three arguments to the three-argument C<open> operator.
+
 For convenience, C<IO::File> exports the O_XXX constants from the
 Fcntl module, if this module is available.
 
@@ -103,7 +108,7 @@ Derived from FileHandle.pm by Graham Barr E<lt>F<gbarr@pobox.com>E<gt>.
 
 =cut
 
-require 5.005_64;
+use 5.006_001;
 use strict;
 our($VERSION, @EXPORT, @EXPORT_OK, @ISA);
 use Carp;
@@ -116,7 +121,7 @@ require Exporter;
 
 @ISA = qw(IO::Handle IO::Seekable Exporter);
 
-$VERSION = "1.08";
+$VERSION = "1.10";
 
 @EXPORT = @IO::Seekable::EXPORT;
 
@@ -157,10 +162,15 @@ sub open {
 	if ($mode =~ /^\d+$/) {
 	    defined $perms or $perms = 0666;
 	    return sysopen($fh, $file, $mode, $perms);
+	} elsif ($mode =~ /:/) {
+	    return open($fh, $mode, $file) if @_ == 3;
+	    croak 'usage: $fh->open(FILENAME, IOLAYERS)';
+	}
+	if (defined($file) && length($file)
+	    && ! File::Spec->file_name_is_absolute($file))
+	{
+	    $file = File::Spec->catfile(File::Spec->curdir(),$file);
 	}
-	if (! File::Spec->file_name_is_absolute($file)) {
-            $file = File::Spec->catfile(File::Spec->curdir(),$file);
-        }
 	$file = IO::Handle::_open_mode_string($mode) . " $file\0";
     }
     open($fh, $file);
