@@ -2,7 +2,7 @@
 #
 # hints file for OpenBSD; Todd Miller <millert@openbsd.org>
 # Edited to allow Configure command-line overrides by
-#  Andy Dougherty <doughera@lafcol.lafayette.edu>
+#  Andy Dougherty <doughera@lafayette.edu>
 #
 # To build with distribution paths, use:
 #	./Configure -des -Dopenbsd_distribution=defined
@@ -11,17 +11,25 @@
 # OpenBSD has a better malloc than perl...
 test "$usemymalloc" || usemymalloc='n'
 
-# Currently, vfork(2) is not a real win over fork(2) but this will
-# change starting with OpenBSD 2.7.
-usevfork='true'
+# malloc wrap works
+case "$usemallocwrap" in
+'') usemallocwrap='define' ;;
+esac
 
-# setre?[ug]id() have been replaced by the _POSIX_SAVED_IDS versions
-# in 4.4BSD.  Configure will find these but they are just emulated
-# and do not have the same semantics as in 4.3BSD.
-d_setregid=$undef
-d_setreuid=$undef
-d_setrgid=$undef
-d_setruid=$undef
+# Currently, vfork(2) is not a real win over fork(2).
+usevfork="$undef"
+
+# In OpenBSD < 3.3, the setre?[ug]id() are emulated using the
+# _POSIX_SAVED_IDS functionality which does not have the same
+# semantics as 4.3BSD.  Starting with OpenBSD 3.3, the original
+# semantics have been restored.
+case "$osvers" in
+[0-2].*|3.[0-2])
+	d_setregid=$undef
+	d_setreuid=$undef
+	d_setrgid=$undef
+	d_setruid=$undef
+esac
 
 #
 # Not all platforms support dynamic loading...
@@ -30,24 +38,35 @@ d_setruid=$undef
 # it can set the libperl name appropriately.
 # Allow command line overrides.
 #
-case "`arch -s`-${osvers}" in
-alpha-2.[0-8]|mips-*|vax-*|powerpc-2.[0-7]|m88k-*)
+ARCH=`arch | sed 's/^OpenBSD.//'`
+case "${ARCH}-${osvers}" in
+alpha-2.[0-8]|mips-2.[0-8]|powerpc-2.[0-7]|m88k-*|hppa-*|vax-*)
 	test -z "$usedl" && usedl=$undef
 	;;
 *)
 	test -z "$usedl" && usedl=$define
-	# we use -fPIC here because -fpic is *NOT* enough for some of the
+	# We use -fPIC here because -fpic is *NOT* enough for some of the
 	# extensions like Tk on some OpenBSD platforms (ie: sparc)
 	cccdlflags="-DPIC -fPIC $cccdlflags"
 	case "$osvers" in
 	[01].*|2.[0-7]|2.[0-7].*)
 		lddlflags="-Bshareable $lddlflags"
 		;;
-	*) # from 2.8 onwards
+	2.[8-9]|3.0)
 		ld=${cc:-cc}
 		lddlflags="-shared -fPIC $lddlflags"
 		;;
+	*) # from 3.1 onwards
+		ld=${cc:-cc}
+		lddlflags="-shared -fPIC $lddlflags"
+		libswanted=`echo $libswanted | sed 's/ dl / /'`
+		;;
 	esac
+
+	# We need to force ld to export symbols on ELF platforms.
+	# Without this, dlopen() is crippled.
+	ELF=`${cc:-cc} -dM -E - </dev/null | grep __ELF__`
+	test -n "$ELF" && ldflags="-Wl,-E $ldflags"
 	;;
 esac
 
@@ -70,10 +89,13 @@ d_suidsafe=$define
 
 # cc is gcc so we can do better than -O
 # Allow a command-line override, such as -Doptimize=-g
-case `arch -s` in
+case ${ARCH} in
 m88k)
    optimize='-O0'
    ;;
+hppa)
+   optimize='-O0'
+   ;;
 *)
    test "$optimize" || optimize='-O2'
    ;;
@@ -87,11 +109,19 @@ $define|true|[yY]*)
 	# any openbsd version dependencies with pthreads?
 	ccflags="-pthread $ccflags"
 	ldflags="-pthread $ldflags"
-	# Add -lpthread.  Also change from -lc to -lc_r
-	libswanted="$libswanted pthread"
-	libswanted=`echo " $libswanted "| sed -e 's/ c / c_r /' -e 's/^ //' -e 's/ $//'`
-	# This is strange.
-	usevfork="$undef"
+	case "$osvers" in
+	[0-2].*|3.[0-2])
+		# Change from -lc to -lc_r
+		set `echo "X $libswanted " | sed 's/ c / c_r /'`
+		shift
+		libswanted="$*"
+	;;
+	esac
+	case "$osvers" in
+	[012].*|3.[0-5])
+        	# Broken at least up to OpenBSD 3.5, we'll see about 3.6
+		d_getservbyname_r=$undef ;;
+	esac
 esac
 EOCBU
 
